/**
 * @Author: Created By McChen
 * @Date: 2017/12/8
 * @Mail: mcchen.club@gmail.com
 * @Version: V1.0.0
 */

export default {
    bind(el, binding, vnode) {
        setTimeout(() => {
            let pos = (el.currentStyle ? el.currentStyle : window.getComputedStyle(el, null)).position;
            if (!pos || pos === 'static') {
                el.style.position = 'relative';
                pos = 'relative';
            }
            if (pos == 'relative' || pos == 'absolute') {
                if (el.children && el.children.length === 1) {
                    let isFF = navigator.userAgent.indexOf('Firefox') > -1;

                    el.children[0].dataset.top = 0;
                    let div = document.createElement('div'), span = document.createElement('span');
                    div.className = 'jrv-inner-scroll-box';
                    span.className = 'jrv-inner-scroll';
                    setTransition(el.children[0], 'all 200ms linear');
                    setTransition(span, 'all 200ms linear');
                    setTransform(el.children[0], 'translate(0,0)');
                    setTransform(span, 'translate(0,0)');
                    div.appendChild(span);
                    el.appendChild(div);

                    let timer = setTimeout(() => {
                        let mousewheel = "mousewheel", dis = 30, boxH = el.clientHeight,
                            childH = el.children[0].clientHeight;
                        if (isFF) { // Firefox浏览器
                            mousewheel = "DOMMouseScroll";
                        }
                        let barH = childH ? Math.round(boxH * boxH / childH) : 0;

                        el.children[0].dataset.top = 0;
                        el.children[0].dataset.childH = childH;
                        if (childH <= boxH) {  // 隐藏滚动条
                            setTransform(el.children[0], 'translate(0,0)');
                            setTransform(span, 'translate(0,0)');
                            span.style.height = '0px';
                        } else {
                            // css3初始化
                            span.style.height = barH + 'px';
                        }

                        el.addEventListener(mousewheel, function (event) {
                            let top = Number(el.children[0].dataset.top) || 0;
                            let _boxH = el.clientHeight, _childH = el.children[0].clientHeight;
                            let _barH = _childH ? Math.round(_boxH * _boxH / _childH) : 0;

                            event.delta = (event.wheelDelta) ? parseInt(event.wheelDelta) / 120 : -(parseInt(event.detail) || 0) / 3;
                            if (event.delta > 0) { // 往上滚
                                let sTop = (top - dis <= 0) ? 0 : (top - dis);
                                el.children[0].dataset.top = sTop;
                                setTransform(el.children[0], 'translate(0,-' + sTop + 'px)');
                                setTransform(span, 'translate(0,' + Math.round(sTop * _barH / _boxH) + 'px)');
                                top = sTop;
                                if (top !== 0) {
                                    event.preventDefault();
                                }
                            } else if (event.delta < 0) { // 往下滚
                                let sTop = (top + dis >= _childH - _boxH) ? (_childH - _boxH) : (top + dis);
                                el.children[0].dataset.top = sTop;
                                setTransform(el.children[0], 'translate(0,-' + sTop + 'px)');
                                setTransform(span, 'translate(0,' + Math.round(sTop * _barH / _boxH) + 'px)');
                                top = sTop;
                                if (top !== _childH - _boxH) {
                                    event.preventDefault();
                                }
                            } else {
                                return;
                            }
                        });

                        clearTimeout(timer);
                    });

                    function setTransform(ele, transform) {
                        ele.style.webkitTransform = ele.style.msTransform = ele.style.transform = transform;
                    }

                    function setTransition(ele, transition) {
                        ele.style.webkitTransition = ele.style.oTransition = ele.style.transition = transition;
                    }
                } else {
                    console.error("There can be only one container child element.")
                }
            } else {
                console.error("Container position should be relative or absolute.")
            }
        }, 0);
    },
    inserted(el, binding, vnode) {
    },
    update(el, binding, vnode, oldVnode) {
        let isFF = navigator.userAgent.indexOf('Firefox') > -1;

        let timer = setTimeout(() => {
            let mousewheel = "mousewheel", boxH = el.clientHeight, childH = el.children[0].clientHeight;
            if (isFF) { // Firefox浏览器
                mousewheel = "DOMMouseScroll";
            }

            let barH = childH ? Math.round(boxH * boxH / childH) : 0;
            let span = el.children[1].children[0];
            if (childH <= boxH) {  // 隐藏滚动条
                span.style.height = '0px';

                el.children[0].dataset.top = 0;
                setTransform(el.children[0], 'translate(0,0)');
                setTransform(span, 'translate(0,0)');
            } else {
                // 滚动条高度变动
                span.style.height = barH + 'px';

                let oldChildH = Number(el.children[0].dataset.childH) || 0; // 老容器高度
                let oldTop = Number(el.children[0].dataset.top) || 0; // 老容器位置
                el.children[0].dataset.childH = childH;

                let changeH = childH - oldChildH; // 高度变动
                if (changeH > 0) { // 高度增加操作
                    // 只需要调整滚动条位置
                    setTransform(span, 'translate(0,' + Math.round(oldTop * barH / boxH) + 'px)');
                } else {  // 高度减少操作
                    if (oldTop >= childH - boxH) { // 置为底部
                        el.children[0].dataset.top = childH - boxH;
                        setTransform(el.children[0], 'translate(0,-' + (childH - boxH) + 'px)');
                        setTransform(span, 'translate(0,' + (boxH - barH) + 'px)');
                    } else {
                        setTransform(span, 'translate(0,' + Math.round(oldTop * barH / boxH) + 'px)');
                    }
                }
            }
            clearTimeout(timer);
        }, 20);


        function setTransform(ele, transform) {
            ele.style.webkitTransform = ele.style.msTransform = ele.style.transform = transform;
        }
    },
    componentUpdated(el, binding, vnode, oldVnode) {
    },
    unbind(el, binding, vnode) {
        let isFF = navigator.userAgent.indexOf('Firefox') > -1;
        let mousewheel = "mousewheel";
        if (isFF) {
            mousewheel = "DOMMouseScroll";
        }
        el.removeEventListener(mousewheel, function (event) {
        })
    }
}